@@ -15,6 +15,7 @@ fn main() {
     let linux = target.contains("unknown-linux");
     let android = target.contains("android");
     let apple = target.contains("apple");
+    let ios = target.contains("apple-ios");
     let emscripten = target.contains("asm");
     let musl = target.contains("musl") || emscripten;
     let uclibc = target.contains("uclibc");
@@ -24,6 +25,7 @@ fn main() {
     let netbsd = target.contains("netbsd");
     let openbsd = target.contains("openbsd");
     let rumprun = target.contains("rumprun");
+    let solaris = target.contains("solaris");
     let bsdlike = freebsd || apple || netbsd || openbsd || dragonfly;
     let mut cfg = ctest::TestGenerator::new();
 
@@ -32,8 +34,14 @@ fn main() {
         cfg.define("_GNU_SOURCE", None);
     } else if netbsd {
         cfg.define("_NETBSD_SOURCE", Some("1"));
+    } else if apple {
+        cfg.define("__APPLE_USE_RFC_3542", None);
     } else if windows {
         cfg.define("_WIN32_WINNT", Some("0x8000"));
+    } else if solaris {
+        cfg.define("_XOPEN_SOURCE", Some("700"));
+        cfg.define("__EXTENSIONS__", None);
+        cfg.define("_LCONV_C99", None);
     }
 
     // Android doesn't actually have in_port_t but it's much easier if we
@@ -77,6 +85,10 @@ fn main() {
             cfg.header("sys/socket.h");
         }
         cfg.header("net/if.h");
+        if !ios {
+            cfg.header("net/route.h");
+            cfg.header("net/if_arp.h");
+        }
         cfg.header("netdb.h");
         cfg.header("netinet/in.h");
         cfg.header("netinet/ip.h");
@@ -94,6 +106,7 @@ fn main() {
         cfg.header("sys/socket.h");
         if linux && !musl {
             cfg.header("linux/if.h");
+            cfg.header("sys/auxv.h");
         }
         cfg.header("sys/time.h");
         cfg.header("sys/un.h");
@@ -103,7 +116,9 @@ fn main() {
         cfg.header("pwd.h");
         cfg.header("grp.h");
         cfg.header("sys/utsname.h");
-        cfg.header("sys/ptrace.h");
+        if !solaris && !ios {
+            cfg.header("sys/ptrace.h");
+        }
         cfg.header("sys/mount.h");
         cfg.header("sys/uio.h");
         cfg.header("sched.h");
@@ -124,6 +139,7 @@ fn main() {
         cfg.header("arpa/inet.h");
         cfg.header("xlocale.h");
         cfg.header("utmp.h");
+        cfg.header("ifaddrs.h");
         if i686 || x86_64 {
             cfg.header("sys/reg.h");
         }
@@ -132,11 +148,11 @@ fn main() {
         cfg.header("ifaddrs.h");
         cfg.header("langinfo.h");
 
-        if !openbsd && !freebsd && !dragonfly {
+        if !openbsd && !freebsd && !dragonfly && !solaris {
             cfg.header("sys/quota.h");
         }
 
-        if !musl && !x32 {
+        if !musl && !x32 && !solaris {
             cfg.header("sys/sysctl.h");
         }
 
@@ -155,25 +171,36 @@ fn main() {
     }
 
     if apple {
+        cfg.header("spawn.h");
         cfg.header("mach-o/dyld.h");
         cfg.header("mach/mach_time.h");
         cfg.header("malloc/malloc.h");
         cfg.header("util.h");
         cfg.header("xlocale.h");
         cfg.header("sys/xattr.h");
-        cfg.header("sys/sys_domain.h");
-        if target.starts_with("x86") {
+        if target.starts_with("x86") && !ios {
             cfg.header("crt_externs.h");
         }
-        cfg.header("net/route.h");
-        cfg.header("net/route.h");
-        cfg.header("sys/proc_info.h");
+        cfg.header("netinet/in.h");
+        cfg.header("sys/ipc.h");
+        cfg.header("sys/shm.h");
+
+        if !ios {
+            cfg.header("sys/sys_domain.h");
+            cfg.header("net/if_utun.h");
+            cfg.header("net/bpf.h");
+            cfg.header("net/route.h");
+            cfg.header("netinet/if_ether.h");
+            cfg.header("sys/proc_info.h");
+            cfg.header("sys/kern_control.h");
+        }
     }
 
     if bsdlike {
         cfg.header("sys/event.h");
-
+        cfg.header("net/if_dl.h");
         if freebsd {
+            cfg.header("net/bpf.h");
             cfg.header("libutil.h");
         } else {
             cfg.header("util.h");
@@ -181,6 +208,7 @@ fn main() {
     }
 
     if linux || emscripten {
+        cfg.header("mntent.h");
         cfg.header("mqueue.h");
         cfg.header("ucontext.h");
         if !uclibc {
@@ -226,24 +254,34 @@ fn main() {
         }
         cfg.header("sys/reboot.h");
         if !emscripten {
+            cfg.header("linux/sockios.h");
+            cfg.header("linux/netlink.h");
+            cfg.header("linux/genetlink.h");
             cfg.header("linux/netfilter_ipv4.h");
+            cfg.header("linux/netfilter_ipv6.h");
             cfg.header("linux/fs.h");
         }
         if !musl {
             cfg.header("asm/mman.h");
-            cfg.header("linux/netlink.h");
             cfg.header("linux/magic.h");
             cfg.header("linux/reboot.h");
+            cfg.header("linux/netfilter/nf_tables.h");
 
             if !mips {
                 cfg.header("linux/quota.h");
             }
         }
     }
+    if solaris {
+        cfg.header("sys/epoll.h");
+    }
 
     if linux || android {
         cfg.header("sys/fsuid.h");
-
+        cfg.header("linux/module.h");
+        cfg.header("linux/seccomp.h");
+        cfg.header("linux/if_ether.h");
+        cfg.header("linux/if_tun.h");
         // DCCP support
         if !uclibc && !musl && !emscripten {
             cfg.header("linux/dccp.h");
@@ -258,9 +296,11 @@ fn main() {
         cfg.header("linux/random.h");
         cfg.header("elf.h");
         cfg.header("link.h");
+        cfg.header("spawn.h");
     }
 
     if freebsd {
+        cfg.header("mqueue.h");
         cfg.header("pthread_np.h");
         cfg.header("sched.h");
         cfg.header("ufs/ufs/quota.h");
@@ -268,9 +308,13 @@ fn main() {
         cfg.header("sys/ipc.h");
         cfg.header("sys/msg.h");
         cfg.header("sys/shm.h");
+        cfg.header("sys/procdesc.h");
+        cfg.header("sys/rtprio.h");
+        cfg.header("spawn.h");
     }
 
     if netbsd {
+        cfg.header("mqueue.h");
         cfg.header("ufs/ufs/quota.h");
         cfg.header("ufs/ufs/quota1.h");
         cfg.header("sys/ioctl_compat.h");
@@ -286,9 +330,18 @@ fn main() {
     }
 
     if dragonfly {
+        cfg.header("mqueue.h");
         cfg.header("ufs/ufs/quota.h");
         cfg.header("pthread_np.h");
         cfg.header("sys/ioctl_compat.h");
+        cfg.header("sys/rtprio.h");
+    }
+
+    if solaris {
+        cfg.header("port.h");
+        cfg.header("ucontext.h");
+        cfg.header("sys/filio.h");
+        cfg.header("sys/loadavg.h");
     }
 
     if linux || freebsd || dragonfly || netbsd || apple || emscripten {
@@ -297,7 +350,7 @@ fn main() {
         }
     }
 
-    cfg.type_name(move |ty, is_struct| {
+    cfg.type_name(move |ty, is_struct, is_union| {
         match ty {
             // Just pass all these through, no need for a "struct" prefix
             "FILE" |
@@ -305,7 +358,15 @@ fn main() {
             "Dl_info" |
             "DIR" |
             "Elf32_Phdr" |
-            "Elf64_Phdr" => ty.to_string(),
+            "Elf64_Phdr" |
+            "Elf32_Shdr" |
+            "Elf64_Shdr" |
+            "Elf32_Sym" |
+            "Elf64_Sym" |
+            "Elf32_Ehdr" |
+            "Elf64_Ehdr" |
+            "Elf32_Chdr" |
+            "Elf64_Chdr" => ty.to_string(),
 
             // Fixup a few types on windows that don't actually exist.
             "time64_t" if windows => "__time64_t".to_string(),
@@ -314,6 +375,10 @@ fn main() {
             // OSX calls this something else
             "sighandler_t" if bsdlike => "sig_t".to_string(),
 
+            t if is_union => {
+                format!("union {}", t)
+            }
+
             t if t.ends_with("_t") => t.to_string(),
 
             // Windows uppercase structs don't have `struct` in front, there's a
@@ -352,9 +417,9 @@ fn main() {
                 }
             }
             "u64" if struct_ == "epoll_event" => "data.u64".to_string(),
-            "type_" if linux &&
+            "type_" if (linux || freebsd || dragonfly) &&
                 (struct_ == "input_event" || struct_ == "input_mask" ||
-                 struct_ == "ff_effect") => "type".to_string(),
+                 struct_ == "ff_effect" || struct_ == "rtprio") => "type".to_string(),
             s => s.to_string(),
         }
     });
@@ -377,8 +442,10 @@ fn main() {
             // which is absent in glibc, has to be defined.
             "__timeval" if linux => true,
 
-            // The alignment of this is 4 on 64-bit OSX...
-            "kevent" if apple && x86_64 => true,
+            // Fixed on stdbuild with repr(packed(4))
+            // Once repr_packed stabilizes we can fix this unconditionally
+            // and remove this check.
+            "kevent" | "shmid_ds" if apple && x86_64 => true,
 
             // This is actually a union, not a struct
             "sigval" => true,
@@ -396,6 +463,17 @@ fn main() {
             // header conflicts when including them with all the other structs.
             "termios2" => true,
 
+            // Present on historical versions of iOS but missing in more recent
+            // SDKs
+            "bpf_hdr" |
+            "proc_taskinfo" |
+            "proc_taskallinfo" |
+            "proc_bsdinfo" |
+            "proc_threadinfo" |
+            "sockaddr_inarp" |
+            "sockaddr_ctl" |
+            "arphdr" if ios => true,
+
             _ => false
         }
     });
@@ -410,7 +488,13 @@ fn main() {
             "uuid_t" if dragonfly => true,
             n if n.starts_with("pthread") => true,
             // sem_t is a struct or pointer
-            "sem_t" if openbsd || freebsd || dragonfly || rumprun => true,
+            "sem_t" if openbsd || freebsd || dragonfly || netbsd => true,
+            // mqd_t is a pointer on FreeBSD and DragonFly
+            "mqd_t" if freebsd || dragonfly => true,
+
+            // Just some typedefs on osx, no need to check their sign
+            "posix_spawnattr_t" |
+            "posix_spawn_file_actions_t" => true,
 
             // windows-isms
             n if n.starts_with("P") => true,
@@ -428,6 +512,8 @@ fn main() {
             "FILE_ATTRIBUTE_INTEGRITY_STREAM" |
             "ERROR_NOTHING_TO_TERMINATE" if mingw => true,
 
+            "SIG_DFL" |
+            "SIG_ERR" |
             "SIG_IGN" => true, // sighandler_t weirdness
             "SIGUNUSED" => true, // removed in glibc 2.26
 
@@ -462,8 +548,17 @@ fn main() {
             "CTL_MAXID" |
             "KERN_MAXID" |
             "HW_MAXID" |
+            "NET_MAXID" |
             "USER_MAXID" if freebsd => true,
 
+            // These constants were added in FreeBSD 11
+            "EVFILT_PROCDESC" | "EVFILT_SENDFILE" | "EVFILT_EMPTY" |
+            "PD_CLOEXEC" | "PD_ALLOWED_AT_FORK" if freebsd => true,
+
+            // These constants were added in FreeBSD 12
+            "SF_USER_READAHEAD" |
+            "SO_REUSEPORT_LB" if freebsd => true,
+
             // These OSX constants are removed in Sierra.
             // https://developer.apple.com/library/content/releasenotes/General/APIDiffsMacOS10_12/Swift/Darwin.html
             "KERN_KDENABLE_BG_TRACE" if apple => true,
@@ -474,9 +569,6 @@ fn main() {
             "KERN_USERMOUNT" |
             "KERN_ARND" if openbsd => true,
 
-            // These constats were added in OpenBSD 6.2
-            "EV_RECEIPT" | "EV_DISPATCH" if openbsd => true,
-
             // These are either unimplemented or optionally built into uClibc
             "LC_CTYPE_MASK" | "LC_NUMERIC_MASK" | "LC_TIME_MASK" | "LC_COLLATE_MASK" | "LC_MONETARY_MASK" | "LC_MESSAGES_MASK" |
             "MADV_MERGEABLE" | "MADV_UNMERGEABLE" | "MADV_HWPOISON" | "IPV6_ADD_MEMBERSHIP" | "IPV6_DROP_MEMBERSHIP" | "IPV6_MULTICAST_LOOP" | "IPV6_V6ONLY" |
@@ -493,7 +585,7 @@ fn main() {
 
             // Defined by libattr not libc on linux (hard to test).
             // See constant definition for more details.
-            "ENOATTR" if linux => true,
+            "ENOATTR" if android || linux => true,
 
             // On mips*-unknown-linux-gnu* CMSPAR cannot be included with the set of headers we
             // want to use here for testing. It's originally defined in asm/termbits.h, which is
@@ -515,6 +607,43 @@ fn main() {
             "BOTHER" => true,
 
             "MFD_CLOEXEC" | "MFD_ALLOW_SEALING" if !mips && musl => true,
+
+            "DT_FIFO" | "DT_CHR" | "DT_DIR" | "DT_BLK" | "DT_REG" | "DT_LNK" | "DT_SOCK" if solaris => true,
+            "USRQUOTA" | "GRPQUOTA" if solaris => true,
+            "PRIO_MIN" | "PRIO_MAX" if solaris => true,
+
+            // These are defined for Solaris 11, but the crate is tested on illumos, where they are currently not defined
+            "EADI" | "PORT_SOURCE_POSTWAIT" | "PORT_SOURCE_SIGNAL" | "PTHREAD_STACK_MIN" => true,
+
+            // These change all the time from release to release of linux
+            // distros, let's just not bother trying to verify them. They
+            // shouldn't be used in code anyway...
+            "AF_MAX" | "PF_MAX" => true,
+
+            // Present on historical versions of iOS, but now removed in more
+            // recent SDKs
+            "ARPOP_REQUEST" |
+            "ARPOP_REPLY" |
+            "ATF_COM" |
+            "ATF_PERM" |
+            "ATF_PUBL" |
+            "ATF_USETRAILERS" |
+            "AF_SYS_CONTROL" |
+            "SYSPROTO_EVENT" |
+            "PROC_PIDTASKALLINFO" |
+            "PROC_PIDTASKINFO" |
+            "PROC_PIDTHREADINFO" |
+            "UTUN_OPT_FLAGS" |
+            "UTUN_OPT_IFNAME" |
+            "BPF_ALIGNMENT" |
+            "SYSPROTO_CONTROL" if ios => true,
+            s if ios && s.starts_with("RTF_") => true,
+            s if ios && s.starts_with("RTM_") => true,
+            s if ios && s.starts_with("RTA_") => true,
+            s if ios && s.starts_with("RTAX_") => true,
+            s if ios && s.starts_with("RTV_") => true,
+            s if ios && s.starts_with("DLT_") => true,
+
             _ => false,
         }
     });
@@ -545,7 +674,7 @@ fn main() {
             "getdtablesize" if android => true,
 
             "dlerror" if android => true, // const-ness is added
-            "dladdr" if musl => true, // const-ness only added recently
+            "dladdr" if musl || solaris => true, // const-ness only added recently
 
             // OSX has 'struct tm *const' which we can't actually represent in
             // Rust, but is close enough to *mut
@@ -566,6 +695,16 @@ fn main() {
             "shm_open" |
             "shm_unlink" |
             "syscall" |
+            "mq_open" |
+            "mq_close" |
+            "mq_getattr" |
+            "mq_notify" |
+            "mq_receive" |
+            "mq_send" |
+            "mq_setattr" |
+            "mq_timedreceive" |
+            "mq_timedsend" |
+            "mq_unlink" |
             "ptrace" |
             "sigaltstack" if rumprun => true,
 
@@ -590,6 +729,9 @@ fn main() {
             // the symbol.
             "uname" if freebsd => true,
 
+            // FIXME: need to upgrade FreeBSD version; see https://github.com/rust-lang/libc/issues/938
+            "setgrent" if freebsd => true,
+
             // aio_waitcomplete's return type changed between FreeBSD 10 and 11.
             "aio_waitcomplete" if freebsd => true,
 
@@ -631,11 +773,31 @@ fn main() {
             // We can wait for the next major release to be compliant with the new API.
             // FIXME: unskip these for next major release
             "strerror_r" | "madvise" | "msync" | "mprotect" | "recvfrom" | "getpriority" |
-            "setpriority" | "personality" if android => true,
+            "setpriority" | "personality" if android || solaris => true,
             // In Android 64 bits, these functions have been fixed since unified headers.
             // Ignore these until next major version.
             "bind" | "writev" | "readv" | "sendmsg" | "recvmsg" if android && (aarch64 || x86_64) => true,
 
+            // signal is defined with sighandler_t, so ignore
+            "signal" if solaris => true,
+
+            "cfmakeraw" | "cfsetspeed" if solaris => true,
+
+            // FIXME: mincore is defined with caddr_t on Solaris.
+            "mincore" if solaris => true,
+
+            // These were all included in historical versions of iOS but appear
+            // to be removed now
+            "system" | "ptrace" if ios => true,
+
+            _ => false,
+        }
+    });
+
+    cfg.skip_static(move |name| {
+        match name {
+            // Internal constant, not declared in any headers.
+            "__progname" if android => true,
             _ => false,
         }
     });
@@ -695,7 +857,8 @@ fn main() {
     // fails on a lot of platforms.
     let mut cfg = ctest::TestGenerator::new();
     cfg.skip_type(|_| true)
-       .skip_fn(|_| true);
+       .skip_fn(|_| true)
+       .skip_static(|_| true);
     if android || linux {
         // musl defines these directly in `fcntl.h`
         if musl {
@@ -721,9 +884,10 @@ fn main() {
         cfg.skip_struct(|s| {
             s != "termios2"
         });
-        cfg.type_name(move |ty, is_struct| {
+        cfg.type_name(move |ty, is_struct, is_union| {
             match ty {
                 t if is_struct => format!("struct {}", t),
+                t if is_union => format!("union {}", t),
                 t => t.to_string(),
             }
         });
